import logging
from optparse import make_option

from django.core.management import BaseCommand
from django.db import transaction

from yx.models import YUser

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--username', dest='username'),
    )

    @transaction.atomic
    def handle(self, username, *args, **options):
        account = YUser.objects.get(username=username)
        synctag(account)


def synctag(account):
    client = account.get_client()
    note_store = client.get_note_store()
    tags = note_store.listTags(account.decrypted_token)

    for tag in tags:
        yxt, _created = account.tags.create_or_update(guid=tag.guid, defaults=dict(name=tag.name.decode('utf-8')))
        logger.info('create tag %s', yxt.guid)



import logging

from evernote.edam.type.ttypes import Note

from yx.models import YNote, YTag, YNotebook, YUser

logger = logging.getLogger(__name__)


def cache_note(yxaccount, note):
    # type: (YUser, Note) -> YNote
    """
    cache note from evernote api into database
    """

    logger.debug('cache_note title=%s', note.title.decode('utf-8'))

    notebookGuid = note.notebookGuid
    notebook, created = YNotebook.objects.get_or_create(
        guid=notebookGuid, defaults=dict(
            yxaccount=yxaccount,
            name=notebookGuid
        ))

    # store into db
    yxnote, _created = YNote.objects.get_or_create(
        guid=note.guid,
        defaults=dict(
            yxaccount=yxaccount,
            notebook=notebook,
        ))
    logger.debug('create note(%s) in database', yxnote.guid)

    yxnote.notebook = notebook
    yxnote.yxaccount = yxaccount

    # copy fields
    yxnote.guid = note.guid
    yxnote.title = note.title
    yxnote.created = note.created
    yxnote.updated = note.updated
    yxnote.attributes = note.attributes

    yxnote.deleted = note.deleted or False
    logger.debug('note deleted=%s', note.deleted)
    yxnote.usn = note.updateSequenceNum

    # note.attributes
    yxnote.reminder_time = note.attributes.reminderTime
    yxnote.reminder_done_time = note.attributes.reminderDoneTime

    yxnote.save()

    # download tags
    newtags = []
    for tagguid in getattr(note, 'tagGuids') or []:
        logger.debug('tagguid=%s', tagguid)
        tag, _created = YTag.objects.get_or_create(guid=tagguid, yxaccount=yxaccount)
        newtags.append(tag)
    yxnote.tags.clear()
    yxnote.tags.add(*newtags)

    return yxnote


def pull_a_tag(account, tag):
    yxtag, created = account.tags.get_or_create(guid=tag.guid)
    yxtag.name = tag.name.decode('utf-8')
    yxtag.notes.clear()
    yxtag.save()

    return tag


def cache_search_result(account, search_result):
    notes = []
    for note in search_result.notes:
        yxnote = cache_note(account, note)
        notes.append(yxnote)
    return notes



import json

from evernote.edam.type.ttypes import Note, NoteAttributes

ATTRS = (
    'subjectDate',
    'latitude',
    'longitude',
    'altitude',
    'author',
    'source',
    'sourceURL',
    'sourceApplication',
    'shareDate',
    'reminderOrder',
    # 'reminderDoneTime',
    # 'reminderTime',
    'placeName',
    'contentClass',
    'applicationData',
    'lastEditedBy',
    'classifications',
    'creatorId',
    'lastEditorId',
)


def NoteattributesToDict(noteattrs):
    result = {key: getattr(noteattrs, key) for key in ATTRS}

    return result


def DictToNoteattributes(dic, noteattrs=None):
    # type: (dict, NoteAttributes) -> NoteAttributes
    noteattrs = noteattrs or NoteAttributes()
    for key in dic:
        setattr(noteattrs, key, dic.get(key))
    return noteattrs


def note_from_modelnote(ynote, note=None):
    note = note or Note()

    note.guid = ynote.guid

    # title can not be starts or ends with space
    note.title = ynote.title.encode('utf-8').strip()

    note.notebookGuid = ynote.notebook.guid
    note.created = ynote.created
    note.updated = ynote.updated
    attrs = NoteAttributes()
    if ynote.attributes:
        m = json.loads(ynote.attributes)
        DictToNoteattributes(m, noteattrs=attrs)  # note.attributes)
    attrs.reminderTime = ynote.reminder_time
    attrs.reminderDoneTime = ynote.reminder_done_time
    attrs.contentClass = ynote.content_class

    note.attributes = attrs

    note.content = ynote.content
    note.deleted = ynote.deleted
    # note.updateSequenceNum=ynote.usn

    # todo: update tags
    # todo: update notebook

    return note


NOTE_ATTR_CONV = {
    'guid': 'guid',
    'title': 'title',
    'created': 'created',
    'updated': 'updated',
    'content_length': 'contentLength',
    'content': 'content',
    # 'reminder_time': 'attributes.reminderTime',
    # 'reminder_done_time': 'attributes.reminderDoneTime',
    # 'content_class': 'attributes.contentClass',
    'deleted': 'deleted',
    'usn': 'updateSequenceNum',
}


def note_to_model_dict(note):
    r = {
        key: getattr(note, key2) for key, key2 in NOTE_ATTR_CONV.iteritems()
    }
    if note.attributes:
        r['attributes'] = NoteattributesToDict(note.attributes)
        r['reminder_time'] = note.attributes.reminderTime
        r['reminder_done_time'] = note.attributes.reminderDoneTime
        r['content_class'] = note.attributes.contentClass
    r['deleted'] = bool(note.deleted)
    return r



import logging
from optparse import make_option

from django.core.management import BaseCommand
from evernote.edam.notestore.ttypes import SyncChunkFilter

from yx.djangohelper import update_or_create
from yx.models import YUser, YNotebook
from yx.utils import cache_note

MAX_ENTITIES = 2 ** 10
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--username'),
        make_option('--fullsync', action='store_true'),
        make_option('--tags', action='store_true'),
        make_option('--notes', action='store_true'),
    )

    def handle(self, username, fullsync, tags, notes, *args, **options):
        logger.debug('username=%s fullsync=%s args=%s, options=%s', username, fullsync, args, options)

        account = YUser.objects.get(username=username)
        devtoken = account.token
        logger.debug('devtoken=%s', devtoken)
        client = account.get_client()
        if fullsync:
            next_usn = 0
        else:
            next_usn = account.last_usn

        # update user profile from yinxiang
        user_store = client.get_user_store()
        user = user_store.getUser(devtoken)
        account.userid_in_evernote = user.id
        account.shardid = user.shardId
        account.save()
        note_store = client.get_note_store()
        fltr = SyncChunkFilter()
        fltr.includeNotes = True
        fltr.includeTags = True
        fltr.includeNoteAttributes = True
        fltr.includeNoteResources = False

        while 1:
            sync_chunk = note_store.getFilteredSyncChunk(devtoken, next_usn, MAX_ENTITIES, fltr)
            if next_usn >= sync_chunk.updateCount:
                break

            logger.debug('chunk usn from %d->%d ==> updateCount=%s,', next_usn, sync_chunk.chunkHighUSN, sync_chunk.updateCount)

            # process sync chunk
            for tag in (sync_chunk.tags or []):
                logging.debug('tag=%s', tag.name.decode('utf-8'))
                yxtag, created = update_or_create(account.tags, guid=tag.guid, defaults=dict(name=tag.name.decode('utf-8')))

            for note in (sync_chunk.notes or []):
                logger.debug('note title=%s', note.title.decode('utf-8'))
                cache_note(account, note)

            for notebook in (sync_chunk.notebooks or []):
                YNotebook.objects.get_or_create(guid=notebook.guid, default=dict(
                    yxaccount=account,
                    name=notebook.name
                ))

            next_usn = sync_chunk.chunkHighUSN

        account.last_usn = next_usn  # sync_chunk.chunkHighUSN
        account.save()
        logger.info('sync completed')



from django.conf import settings
from evernote.api.client import EvernoteClient


def get_anonymous_client():
    client = EvernoteClient(consumer_key=settings.EVERNOTE_CONSUMER_KEY,
                            consumer_secret=settings.EVERNOTE_CONSUMER_SECRET,
                            service_host=settings.EVERNOTE_SERVICE_HOST)
    return client



import logging

from enum import IntEnum
from evernote.edam.notestore.ttypes import SyncChunkFilter

from yx.models import SyncMark, YUser
from yx.sync.exceptions import ConflictError
from yx.sync.notebooks import cache_notebook
from yx.sync.notes import push_note_create, push_note_update, save_note_in_local_db
from yx.sync.tags import pull_a_tag

MAX_ENTITIES = 2 ** 10


class ConflictOption(IntEnum):
    RAISE_EXCEPTION = 0
    FORCE_OVERWRITE = 1


logger = logging.getLogger('yx.sync')


def pull(username, fullsync=None, conflict_option=ConflictOption.FORCE_OVERWRITE, **options):
    # logger.debug('databases=%s', settings.DATABASES)
    logger.debug('username=%s fullsync=%s  options=%s', username, fullsync, options)
    # logger.debug('all=%s', YUser.objects.all())
    account = YUser.objects.get(username=username)
    devtoken = account.token
    client = account.get_client()
    if fullsync:
        next_usn = 0
    else:
        next_usn = account.last_usn

    # pull user info #################################
    user_store = client.get_user_store()
    user = user_store.getUser(devtoken)
    account.userid_in_evernote = user.id
    account.shardid = user.shardId
    account.save()

    # pull chunk #################################
    note_store = client.get_note_store()
    fltr = SyncChunkFilter(
        includeNotebooks=True,
        includeNotes=True,
        includeTags=True,
        includeNoteAttributes=True,
    )
    while 1:
        sync_chunk = note_store.getFilteredSyncChunk(devtoken, next_usn, MAX_ENTITIES, fltr)
        # If updateCount
        # and chunkHighUSN are identical, that means that this is the last chunk
        # in the account ... there is no more recent information.
        if next_usn >= sync_chunk.updateCount:
            break

        logger.debug('------- process (%s,%s) of %s updates', next_usn, sync_chunk.chunkHighUSN, sync_chunk.updateCount)

        # pull tags #################################
        logger.debug('--------- pull tags --------')
        for tag in (sync_chunk.tags or []):
            yxtag = pull_a_tag(account, tag)

        # pull notebooks #################################
        logger.debug('--------- pull notebooks --------')
        for notebook in (sync_chunk.notebooks or []):
            ynb = cache_notebook(notebook, account)

        # pull note info #################################
        logger.debug('--------- pull notes --------')
        for note in (sync_chunk.notes or []):
            logger.debug('note title=%s', note.title.decode('utf-8'))
            ynote = save_note_in_local_db(note, account, conflict_option)

        next_usn = sync_chunk.chunkHighUSN

    account.last_usn = next_usn  # sync_chunk.chunkHighUSN
    account.save()
    logger.info('sync completed')


def push(username, fullsync=None, conflict_option=ConflictOption.RAISE_EXCEPTION, **options):
    account = YUser.objects.get(username=username)
    token = account.token
    lastusn = account.last_usn

    client = account.get_client()
    note_store = client.get_note_store()

    # push tags
    for tag in account.tags.filter(sync_mark=SyncMark.NEW):
        # create tag
        pass

    for tag in account.tags.filter(sync_mark=SyncMark.UPDATE):
        # update tag
        pass

    # push notebooks
    for tag in account.notebooks.filter(sync_mark=SyncMark.NEW):
        # create tag
        pass

    for tag in account.notebooks.filter(sync_mark=SyncMark.UPDATE):
        # update tag
        pass

    # push new notes
    for ynote in account.notes.filter(sync_mark=SyncMark.NEW):
        logger.info('push to create note "%s"', ynote.title)
        push_note_create(ynote, note_store, token)

    for ynote in account.notes.filter(sync_mark=SyncMark.UPDATE):
        logger.info('push to update note "%s"', ynote.title)
        try:
            push_note_update(ynote, note_store, token, conflict_option)
        except ConflictError:
            # todo : mark note conflicted
            logger.warning('note with guid(%s) conflicted', ynote.guid)
            continue


def pullsinglenote(guid, **options):
    account = YUser.objects.get(token=options['token'])
    client = account.get_client()
    note_store = client.get_note_store()

    # note = Yxnote.objects.get(guid=guid)

    # update note content with resource data

    # download resources
    notedata = note_store.getNote(
        account.token, guid,
        True,  # withContent
        False,  # withResourcesData
        False,  # withResourcesRecognition
        False  # withResourcesAlternateData
    )

    save_note_in_local_db(notedata, account, ConflictOption.FORCE_OVERWRITE)

    # todo: download resource, but not now
    # if hasattr(notedata, 'resources') and notedata.resources:
    #     for res in notedata.resources:
    #         logger.debug('res=%s', res)
    #         # update res usn
    #         # write res data into file
    #         fn = binascii.hexlify(res.data.bodyHash)
    #         with open(os.path.join(BASE_DIR, 'files', fn), 'wb') as o:
    #             o.write(res.data.body)

    # convert note content to markdown, and store into db
    # note.save()



import logging

from yx.djangohelper import update_or_create
from yx.models import YUser

logger = logging.getLogger('yx.sync')


def pullalltags(username, *args, **options):
    account = YUser.objects.get(username=username)
    client = account.get_client()
    note_store = client.get_note_store()
    tags = note_store.listTags(account.token)

    for tag in tags:
        yxt, _created = update_or_create(
            account.tags,
            guid=tag.guid,
            defaults=dict(
                name=tag.name.decode('utf-8')
            )
        )
        logger.info('create tag %s', yxt.guid)


def pull_a_tag(account, tag):
    logger.debug('save tag "%s"', tag.name.decode('utf-8'))
    yxtag, created = update_or_create(
        account.tags,
        guid=tag.guid,
        defaults=dict(
            name=tag.name.decode('utf-8'),
            usn=tag.updateSequenceNum,
        )
    )
    yxtag.set_sync_mark_to_normal()  # yxtag.sync_mark = SyncMark.NORMAL
    yxtag.name = tag.name.decode('utf-8')
    yxtag.notes.clear()
    yxtag.save()

    return yxtag



from django.core.exceptions import ObjectDoesNotExist


def update_or_create(model_manager, defaults=None, **kwargs):
    defaults = defaults or {}
    try:
        instance = model_manager.get(**kwargs)
        for k, v in defaults.items():
            setattr(instance, k, v)
        instance.save()
        return instance, False
    except ObjectDoesNotExist:
        defaults.update(kwargs)
        instance = model_manager.create(**defaults)
        return instance, True



from optparse import make_option

from django.core.management import BaseCommand

from yx.sync.fullsync import pullsinglenote


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--token', dest='token'),
        make_option('--guids', nargs='+'),
    )

    def handle(self, *guids, **options):
        for guid in guids:
            pullsinglenote(guid, **options)



import logging
from optparse import make_option

from django.core.management import BaseCommand
from django.db import transaction

from yx.models import YUser
from yx.utils import cache_note

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--token', dest='token'),
    )

    @transaction.atomic
    def handle(self, *guids, **options):
        account = YUser.objects.get(token=options['token'])
        client = account.get_client()
        note_store = client.get_note_store()

        for guid in guids:
            # note = Yxnote.objects.get(guid=guid)

            # update note content with resource data

            # download resources
            notedata = note_store.getNote(
                account.decrypted_token, guid,
                True,  # withContent
                False,  # withResourcesData
                False,  # withResourcesRecognition
                False  # withResourcesAlternateData
            )

            cache_note(account, notedata)

        # fixme: do not download resource now
        # if hasattr(notedata, 'resources') and notedata.resources:
        #     for res in notedata.resources:
        #         logger.debug('res=%s', res)
        #         # update res usn
        #         # write res data into file
        #         fn = binascii.hexlify(res.data.bodyHash)
        #         with open(os.path.join(BASE_DIR, 'files', fn), 'wb') as o:
        #             o.write(res.data.body)

        # convert note content to markdown, and store into db
        # note.save()



# a search
from evernote.edam.notestore.ttypes import NoteFilter, NotesMetadataResultSpec
from evernote.edam.type.ttypes import NoteSortOrder


class Search(object):

    def __init__(self, yuser, words):
        self.yuser = yuser
        self.words = words

    def search(self):
        # words = words.encode('utf-8') if isinstance(words, unicode) else words

        client = self.yuser.get_client()  # yxappsettings.get_client(token=self.decrypted_token)

        note_store = client.get_note_store()

        fltr = NoteFilter()
        fltr.words = self.words.encode('utf-8')
        fltr.order = NoteSortOrder.CREATED
        fltr.ascending = True

        spec = NotesMetadataResultSpec()
        spec.includeTitle = True
        spec.includeCreated = True
        spec.includeUpdateSequenceNum = True
        spec.includeTagGuids = True

        search_result = note_store.findNotesMetadata(self.yuser.decrypted_token, fltr, 0, 5000, spec)

        return search_result



from __future__ import absolute_import

# from .management.commands.pull import pull_change
# from .models import Resource

# logger = logging.getLogger(__name__)


# @login_required()
# def xsync(request, username):
# 	acc = get_object_or_404(YxAccount, username=username)
# 	pull_change(acc, refresh_sync=False)
#
# 	nxt = request.GET.get('next') or 'index'
# 	return redirect(nxt)


# @login_required
# def get_notes(request, username):
# 	user = get_object_or_404(YxAccount, username=username)

# 	# daterange = request.GET.get('daterange')
# 	# filters = {}
# 	# if daterange == 'today':
# 	#     filters['updated__gte'] = 0
# 	#     filters['updated__lte'] = 1

# 	notes = user.notes.all().order_by('created')
# 	return render(request, 'notes/index.html', {
# 		# 'title': daterange,
# 		'notes': notes,
# 	})


# @login_required
# def list_all_objectives(request, username):
# 	account = get_object_or_404(get_user_model(), username=username)
# 	obj_tag = get_object_or_404(account.tags, name='objective')
# 	objs = obj_tag.notes.all()

# 	orderfield = request.GET.get('orderfield', 'created')
# 	objs = objs.order_by(orderfield)

# 	return render(request, 'okr/index.html', {'title': 'Objectives', 'objectives': objs})


# @login_required
# def get_tasklist(request, username):
# 	acc = get_object_or_404(get_user_model(), username=username)
# 	tasktag = get_object_or_404(acc.tags, name='task')
# 	tasks = tasktag.notes.all().order_by('reminder_time', 'created')
# 	return render(request, 'tasks/tasks-list.html', {'title': 'All tasks', 'tasks': tasks})


# @login_required()
# def list_all_tags(request, username):
# 	user = get_object_or_404(get_user_model(), username=username)
# 	query = user.tags.all().annotate(
# 		note_count=Count('notes'), last_updated=Max('notes__created')).order_by('-last_updated')

# 	if not settings.DEBUG:
# 		logger.debug('exclude note_count=0 in non-debug')
# 		query = query.exclude(note_count=0)

# 	return render(request, 'tags/index.html', {
# 		'tags': query,
# 	})


# @login_required
# def get_shopping_list(request, username):
# 	acc = get_object_or_404(YxAccount, username=username)
# 	shopping_tag = get_object_or_404(acc.tags, name='shopping')
# 	items = shopping_tag.notes.all()
# 	return render(request, 'shopping/index.html', {'title': 'Shopping List', 'items': items})


# @login_required
# def list_all_books(request, username):
# 	acc = get_object_or_404(get_user_model(), username=username)
# 	booktag = get_object_or_404(acc.tags, name='book')
# 	books = booktag.notes.all()
# 	return render(request, 'books/index.html', {'title': 'Books', 'books': books})


# @login_required()
# def view_tagged_notes(request, username, tagnames):
# 	user = get_object_or_404(get_user_model(), username=username)
# 	notes = []
# 	tags = set()
# 	try:
# 		nameset = set(tagnames.split('/'))
# 		tags = set((tag for tag in user.tags.filter(name__in=nameset)))
# 		if tags:
# 			notes = user.notes.filter(tags__in=tags)
# 	except ObjectDoesNotExist:
# 		pass
# 	return render(request, 'notes/index.html', {'notes': notes})


# @login_required
# def synctags(request, username):
# 	acc = get_object_or_404(get_user_model(), username=username)
# 	synctag(acc)
# 	url = reverse('list-all-tags', args=(username,))
# 	return redirect(url)


# @login_required
# def search(request, username):
# 	notes = []
# 	if request.method == 'POST':
# 		form = EnSelectForm(request.POST)
# 		if form.is_valid():
# 			account = get_object_or_404(YxAccount, username=username)
# 			client = account.get_client()
# 			note_store = client.get_note_store()

# 			note_filter = NoteFilter()
# 			note_filter.words = form.cleaned_data['words']  # 'tag:task'
# 			note_filter.order = NoteSortOrder.CREATED
# 			note_filter.ascending = True

# 			spec = NotesMetadataResultSpec()
# 			spec.includeTitle = True
# 			spec.includeCreated = True
# 			spec.includeUpdateSequenceNum = True
# 			spec.includeTagGuids = True
# 			spec.includeAttributes = True

# 			search_result = note_store.findNotesMetadata(account.decrypted_token, note_filter, 0, 5000, spec)
# 			notes = cache_search_result(account, search_result)
# 	else:
# 		form = EnSelectForm()

# 	return render(request, 'yx/search.html', {
# 		'form': form,
# 		'notes': notes,
# 	})


# @login_required
# def viewmeta(request, username, guid):
# 	account = get_object_or_404(YxAccount, username=username)
# 	client = account.get_client()
# 	note_store = client.get_note_store()
# 	yxnote = note_store.getNote(account.decrypted_token, guid,
# 								True,  # content
# 								True,  # res data
# 								False,  # reco
# 								False)  # alter
# 	root = etree.fromstring(yxnote.content)
# 	hash_to_refcount = {}
# 	for media_elm in root.iter('en-media'):
# 		hash = media_elm.attrib['hash']
# 		try:
# 			hash_to_refcount[hash] += 1
# 		except KeyError:
# 			hash_to_refcount[hash] = 1
# 	if yxnote.resources:
# 		for res in yxnote.resources:
# 			res.hash = binascii.hexlify(res.data.bodyHash)
# 			res.refcount = hash_to_refcount.get(res.hash, 0)
# 	return render(request, 'notes/meta.html', {
# 		'note': yxnote,
# 		'resources': yxnote.resources,
# 		'attribute': yxnote.attributes,
# 	})


# @login_required
# def thumbnail(request, username, guid):
# 	filepath = os.path.join(settings.THUMBNAIL_DIR, guid + '.png')
# 	if not os.path.isfile(filepath):
# 		account = get_object_or_404(YxAccount, username=username)
# 		note = get_object_or_404(account.notes, guid=guid)  # check note exists

# 		# post to get
# 		rsp = requests.post(note.thumbnail_url, {'auth': account.decrypted_token})
# 		if rsp.status_code == 200:
# 			try:
# 				os.makedirs(settings.THUMBNAIL_DIR)
# 			except OSError:
# 				pass
# 			open(filepath, 'wb').write(rsp.content)
# 		else:
# 			raise Http404

# 	if os.path.isfile(filepath):
# 		data = open(filepath, 'rb').read()
# 	else:
# 		placeholder_path = os.path.join(os.path.dirname(__file__), 'placeholder.png')
# 		data = open(placeholder_path, 'rb').read()
# 	return HttpResponse(data, content_type='image/png')


# @login_required
# def resource(request, username, reshash):
# 	user = get_object_or_404(YxAccount, username=username)
# 	res = get_object_or_404(Resource, note__yxaccount=user)
# 	mime = res.mime
# 	try:
# 		fp = os.path.join(settings.BASE_DIR, 'files', reshash)
# 		data = open(fp, 'rb').read()
# 	except IOError:  # file not exist
# 		# todo: download it
# 		fp = os.path.join(os.path.dirname(__file__), 'placeholder.png')
# 		data = open(fp, 'rb').read()
# 		mime = 'image/png'
# 	return HttpResponse(data, content_type=mime)



from optparse import make_option

from django.core.management import BaseCommand

from yx.sync.tags import pullalltags


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--username', dest='username'),
    )

    def handle(self, **options):
        return pullalltags(**options)



import json
import os
import unittest

from django.conf import settings
from django.test import Client

TEST_RESULTS_DIR = os.path.join(settings.BASE_DIR, 'test_results')


class HttpException(BaseException):
    def __init__(self, rsp):
        super(HttpException, self).__init__()
        self.rsp = rsp

    def __str__(self):
        return '<Response: %d>' % self.rsp.status_code


class TestBase(unittest.TestCase):
    """
    Base class for django unit test
    forbid copy of database
    """

    def __init__(self, *args, **kwargs):
        super(TestBase, self).__init__(*args, **kwargs)
        self._client = Client()  # use django client

    def _call_http_method(self, method, path, data=None):
        data = data or {}
        self._save_url(data, path, method, '-request')

        jsonstring = json.dumps(data)
        client_method = getattr(self._client, method.lower())
        rsp = client_method(path, data=jsonstring, content_type='application/json')
        if rsp.status_code != 200:
            raise HttpException(rsp)
        self._save_url(rsp.content, path, method)
        return rsp

    def _save_url(self, s, path, method, suffix='-response'):
        if isinstance(s, dict):
            s = json.dumps(s)

        test_root = TEST_RESULTS_DIR
        fdir = test_root + '/' + path  # os.path.join(test_root, path)
        if not os.path.isdir(fdir):
            os.makedirs(fdir)
        filepath = os.path.join(fdir, method + suffix + '.json')
        open(filepath, 'wb').write(s)

    def get(self, path, data=None):
        data = data or {}
        self._save_url(data, path, 'get', suffix='-request')
        rsp = self._client.get(path, data)  ####use origin
        if rsp.status_code != 200:
            raise HttpException(rsp)
        self._save_url(rsp.content, path, 'get', suffix='-response')
        return rsp

    def post(self, url, data=None, data_format='json'):
        data_format = data_format.lower()
        if data_format == 'json':
            return self._call_http_method(method='post', path=url, data=data)
        # multiform
        self._save_url(data, url, 'post', suffix='-request')
        rsp = self._client.post(url, data)
        if rsp.status_code != 200:
            raise HttpException(rsp)
        self._save_url(rsp.content, url, 'post')
        return rsp

    def put(self, url, data=None):
        return self._call_http_method(method='put', path=url, data=data)

    def patch(self, url, data=None):
        return self._call_http_method(method='patch', path=url, data=data)

    def delete(self, url, data=None):
        return self._call_http_method(method='delete', path=url, data=data)



import logging

from yx.djangohelper import update_or_create
from yx.models import YNotebook

logger = logging.getLogger('yx.sync')


def cache_notebook(notebook, account):
    logger.debug('notebook=%s', notebook.name.decode('utf-8'))
    ynb, create = update_or_create(
        YNotebook.objects,
        guid=notebook.guid,
        defaults=dict(
            name=notebook.name.decode('utf-8'),
            usn=notebook.updateSequenceNum,
        )
    )
    ynb.set_sync_mark_to_normal()  # ynb.sync_mark = SyncMark.NORMAL
    ynb.name = notebook.name.decode('utf-8')
    ynb.usn = notebook.updateSequenceNum
    ynb.save()

    return ynb



import logging

from django.core.exceptions import ObjectDoesNotExist
from evernote.edam.type.ttypes import Note

from yx.djangohelper import update_or_create
from yx.models import SyncMark, YNote, YNotebook, YUser
from .exceptions import ConflictError
from .fullsync import ConflictOption
from .typeconverters import note_from_modelnote, note_to_model_dict

logger = logging.getLogger(__name__)


def save_note_in_local_db(note, yxaccount, conflict_option):
    # type: ( Note, YUser,ConflictOption) -> YNote
    """
    cache note from evernote api into database
    """

    logger.debug('cache_note title=%s', note.title.decode('utf-8'))

    # check usn now
    try:
        ynote = yxaccount.notes.get(guid=note.guid)
        if ynote.usn != note.updateSequenceNum:
            logger.warning('conflict occurred, force overwrite for %s', note.title.decode('utf-8'))

            if conflict_option == ConflictOption.RAISE_EXCEPTION:  # otherwise
                raise ConflictError
            elif conflict_option == ConflictOption.FORCE_OVERWRITE:
                pass

    except ObjectDoesNotExist:
        pass

    notebookGuid = note.notebookGuid
    notebook, created = update_or_create(
        YNotebook.objects,
        guid=notebookGuid,
        defaults=dict(
            yxaccount=yxaccount,
            name=notebookGuid
        )
    )

    d = note_to_model_dict(note)

    # store into db
    ynote, _created = update_or_create(
        yxaccount.notes,
        guid=note.guid,
        defaults=dict(
            notebook=notebook,
            sync_mark=SyncMark.NORMAL,
            **d
        )
    )
    logger.debug('create note(%s) in database', ynote.guid)

    # update note tags
    newtags = []
    for tag_guid in note.tagGuids or []:
        logger.debug('tag_guid=%s', tag_guid)
        tag, _created = update_or_create(
            yxaccount.tags,
            guid=tag_guid,
            defaults=dict(
                name=tag_guid,
                usn=0,
                sync_mark=SyncMark.NORMAL
            )
        )
        newtags.append(tag)
    ynote.tags.clear()
    ynote.tags.add(*newtags)

    return ynote


def push_note_create(ynote, note_store, token):
    note = note_from_modelnote(ynote)

    # if note.sync_mark == SyncMark.NEW:
    note = note_store.createNote(token, note)

    ynote.set_sync_mark_to_normal()  # = SyncMark.NORMAL
    ynote.guid = note.guid
    ynote.usn = note.updateSequenceNum
    ynote.save()

    return ynote


def push_note_update(ynote, note_store, token, conflict_option):
    # elif note.sync_mark == SyncMark.UPDATE:
    origin_note = note_store.getNote(
        token,
        ynote.guid,
        False,  # withContent
        False,  # withResourcesData
        False,  # withResourcesRecognition
        False  # withResourcesAlternateData
    )
    if ynote.usn != origin_note.updateSequenceNum:
        logger.warning('push note %s with conflict', ynote.title)
        if conflict_option == ConflictOption.RAISE_EXCEPTION:
            raise ConflictError
        elif conflict_option == ConflictOption.FORCE_OVERWRITE:
            pass

    if ynote.deleted:
        logger.debug('deleting note %s', origin_note.title.decode('utf-8'))
        updateSequenceNum = note_store.deleteNote(token, ynote.guid)
    else:
        logger.debug('update note %s to %s', origin_note.title.decode('utf-8'), ynote.title)
        note_from_modelnote(ynote, note=origin_note)
        note = note_store.updateNote(token, origin_note)
        updateSequenceNum = note.updateSequenceNum

    ynote.set_sync_mark_to_normal()  # ynote.sync_mark = SyncMark.NORMAL
    ynote.usn = updateSequenceNum
    ynote.save()

    return ynote



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt', 'r') as requ:
    install_requires = [line.strip() for line in requ if line.strip()]

setuptools.setup(
    name='django_yx_app',
    version="0.0.3",
    author="yijie zeng",
    author_email="axplus@163.com",
    description="evernote helper app of django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/axplus/django_yx_app",
    packages=['yx', 'yx.sync', 'yx.management', 'yx.management.commands'],
    classifiers=[
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=2.6',
    install_requires=install_requires,
# include_package_data=True,
)



from django.contrib import admin

from .models import YNote, YTag, YNotebook, YUser


class NoteAdmin(admin.ModelAdmin):
    pass


admin.site.register(YNote, NoteAdmin)


class TagAdmin(admin.ModelAdmin):
    pass


admin.site.register(YTag, TagAdmin)


class YxAccountAdmin(admin.ModelAdmin):
    pass


admin.site.register(YUser, YxAccountAdmin)


class YNotebookAdmin(admin.ModelAdmin):
    pass


admin.site.register(YNotebook, YNotebookAdmin)



from yx.sync.fullsync import ConflictOption
from yx.sync.notes import save_note_in_local_db


def cache_search_result(search_result, account):
    notes = []
    for note in search_result.notes:
        yxnote = save_note_in_local_db(note, account, ConflictOption.FORCE_OVERWRITE)
        notes.append(yxnote)
    return notes



import logging
import time

from django.conf import settings
from django.contrib.auth.models import AbstractUser, User
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from evernote.api.client import EvernoteClient

INAPPLINKFORMAT = 'evernote:///view/{userId}/{shardId}/{noteGuid}/{noteGuid}/'
HTTPLINKFORMAT = 'https://{service}/shard/{shardId}/nl/{userId}/{noteGuid}/'
NOTE_THUMBNAIL_URL_FORMAT = 'https://{host}/shard/{shardId}/thm/note/{GUID}'

logger = logging.getLogger(__name__)


def _nanoseconds_now():
    return int(time.time() * 1000)


class YUserManager(models.Manager):

    def get_anonymous_client(self):
        client = EvernoteClient(consumer_key=settings.EVERNOTE_CONSUMER_KEY,
                                consumer_secret=settings.EVERNOTE_CONSUMER_SECRET,
                                service_host=settings.EVERNOTE_SERVICE_HOST)
        return client


class YUser(models.Model):
    user = models.OneToOneField(User, related_name='+', on_delete=models.CASCADE)

    username = models.CharField(max_length=32)
    token = models.TextField(null=True, blank=True)
    userid_in_evernote = models.IntegerField(default=0)
    shardid = models.CharField(max_length=10, default='s0')
    last_usn = models.IntegerField(default=0)

    objects = YUserManager()

    def __unicode__(self):
        return '%s@%s' % (self.username, settings.EVERNOTE_SERVICE_HOST)

    def get_client(self):
        # type: () -> EvernoteClient
        client = EvernoteClient(token=self.token,
                                service_host=settings.EVERNOTE_SERVICE_HOST)

        return client


class YTag(models.Model):
    yxaccount = models.ForeignKey('YUser', related_name='tags')
    notes = models.ManyToManyField('YNote', related_name='tags')
    guid = models.CharField(max_length=100)
    name = models.CharField(max_length=100)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('name',)


class YNotebook(models.Model):
    yxaccount = models.ForeignKey('YUser', related_name='notebooks')
    name = models.CharField(max_length=100)
    guid = models.CharField(max_length=100)

    def __unicode__(self):
        return self.name


class YNote(models.Model):
    yxaccount = models.ForeignKey('YUser', related_name='notes', on_delete=models.PROTECT)
    guid = models.CharField(max_length=100, unique=True)
    notebook = models.ForeignKey('YNotebook', related_name='notes')
    title = models.CharField(max_length=1000)
    created = models.BigIntegerField(default=_nanoseconds_now)
    updated = models.BigIntegerField(default=_nanoseconds_now)
    content = models.TextField(null=True, blank=True)
    content_length = models.IntegerField(default=0)
    attributes = models.TextField(default='', blank=True)
    reminder_time = models.BigIntegerField(blank=True, null=True)
    reminder_done_time = models.BigIntegerField(blank=True, null=True)

    usn = models.IntegerField(default=0)

    deleted = models.BooleanField(default=False)

    def __unicode__(self):
        if self.deleted:
            return '<strike>%s</strike>' % self.title

        return self.title

    def tagged(self, *tagnames):
        try:
            return self.tags.filter(name__in=tagnames).count() == len(tagnames)
        except ObjectDoesNotExist:
            pass

    @property
    def link_in_app(self):
        return INAPPLINKFORMAT.format(userId=self.yxaccount.userid_in_evernote,
                                      shardId=self.yxaccount.shardid,
                                      noteGuid=self.guid)

    class Meta:
        ordering = ('-updated',)
